<?php

class ImageGalleryPage extends GalleryPage
{
	/**
	 * Same as GalleryPage, only one more option for ordering by Sort.
	 */
	static $db = array (
		'SortBy' => "Enum( array( 'Title', 'UploadDateASC', 'UploadDateDESC','SortASC' ), 'SortASC' )",
		'GalleryUI' => "Enum( array( 'Lightbox','Lightbox (alt)','Shadowbox','Thickbox','Fancybox','PrettyPhoto','NyroModal'),'Lightbox')",
		'CoverImageWidth' => 'Int',
		'CoverImageHeight' => 'Int'
	);
	
	static $has_one = array (
		'RootFolder' => 'Folder'
	);
	
	static $defaults = array (
		'CoverImageWidth' => '100',
		'CoverImageHeight' => '100',
		'ThumbnailWidth' => '85',
		'ThumbnailHeight' => '85',
		'NormalWidth' => '500',
		'NormalHeight' => '500',
		'NormalType' => 'FittedResize',
		'ThumbnailType' => 'CroppedResize',
		'MediaPerPage' => '30',
		'MediaPerLine' => '6',
		'SortBy' => 'SortASC'
	);
	
	static $has_many = array (
		'Albums' => 'ImageGalleryAlbum'
	);
	
	static $attrs = array (
		'Shadowbox' => array(	
			'rel' => 'shadowbox[gallery]',
			'class' => 'shadowbox'
		),
		'Lightbox' => array (
			'rel' => 'lightbox',
			'class' => 'lightbox'
		),
		'Lightbox (alt)' => array(
			'rel' => 'lightbox-gallery',
			'class' => 'lightbox'
		),
		'Thickbox' => array (
			'rel' => 'thickbox-gallery',
			'class' => 'thickbox'
		),
		'Fancybox' => array (
			'rel' => 'fancybox',
			'class' => 'fancybox'
		),
		'PrettyPhoto' => array (
			'rel' => 'prettyPhoto[gallery]',
			'class' => 'prettyPhoto'		
		),
		'NyroModal' => array (
			'rel' => 'gal',
			'class' => 'nyroModal'
		)
	);
	
	
	public function requireDefaultRecords()
	{
		parent::requireDefaultRecords();
		$folder = Folder::findOrMake("ImageGallery");
		$folder->Title = "Image Gallery";
		$folder->write();
		
	}
	
	private function requireDefaultAlbum()
	{
		if(!$this->Albums()->Count()) {
			$album = new ImageGalleryAlbum();
			$album->Title = "Default Album";
			$album->Source = "new";			
			$album->ImageGalleryPageID = $this->ID;
			$album->write();
		}
			
	}
	
	public function getCMSFields($cms) {
		$f = parent::getCMSFields($cms);
		$f->replaceField( 'SortBy' , new OptionsetField(
			'SortBy',
			'Sorted By',
			array(
				'Title' => 'Title',
				'UploadDateASC' => 'Upload Date Ascending',
				'UploadDateDESC' => 'Upload Date Descending',
				'SortASC' => 'Allow Drag and Drop Ordering'
			)
		));
		$f->removeByName('ShowTitle');
		$f->removeByName('ShowSize');
		$f->removeByName('ShowImageDimensions');
		$f->removeByName('FolderID');
		$f->removeByName('DefaultIcon');
		
		$f->addFieldToTab("Root.Content.Popup",new OptionsetField('GalleryUI','Popup style', singleton('ImageGalleryPage')->dbObject('GalleryUI')->enumValues()));
		
		$f->removeFieldFromTab('Root.Content.Extensions','Extensions');
		$f->addFieldToTab('Root.Content.Extensions', new HeaderField($title ='This page type only allows jpg or jpeg file types.',$headingLevel='2'));
		
		$this->requireDefaultAlbum();
		
		$table = new ComplexTableField(
			$this,
			'Albums',
			'ImageGalleryAlbum',
			array('Title' => 'Title'),
			'getCMSFields_forPopup',
			"ImageGalleryPageID = {$this->ID}"
		);
		
		
		
		
		$f->addFieldToTab("Root.Content.Albums", $table);
		$f->addFieldToTab("Root.Content.Albums", new HeaderField($title = "Album cover images", $headingLevel = "3"));
		$f->addFieldToTab("Root.Content.Albums", new FieldGroup(
				new NumericField('CoverImageWidth','Width'),
				new NumericField('CoverImageHeight','Height')
			)
		);
		return $f;
	}
	
	/**
	 * Same as GalleryPage, only adds a handler for the new Sort ASC option.
	 */
	function Items( $limit = "" ) {

		// TODO Implement this, searching through sub-directories if required to find appropriate files
		
		if($current_album = $this->CurrentAlbum()) {
			$folder = $current_album->Folder();
		/** Build the list of allowed file extensions **/
			if( $this->SortBy == 'Title' )
				$sort = "`File`.`Title` ASC";
			else if( $this->SortBy == 'UploadDateASC' )
				$sort = "`File`.`Created` ASC";
			else if( $this->SortBy == 'UploadDateDESC' )
				$sort = "`File`.`Created` DESC";
			else if ( $this->SortBy == 'SortASC' )
				$sort = "`File`.`Sort` ASC";
			$files = DataObject::get(
				"File", 
				"ParentID = {$folder->ID}", 
				$sort, 
				"", 
				$limit
			);
			if($files) {
				foreach($files as $file) {
					$file->RelAttr = self::$attrs[$this->GalleryUI]['rel'];
					$file->ClassAttr = self::$attrs[$this->GalleryUI]['class'];
				}
			}
			return $files;
		}
		else
			return null;
	}


	function GalleryItems( $limit ) {
		
		/** Get the items from the database **/
		$items = $this->Items( $limit );

		/** For every item that matches the filter... **/
		if( $items ) {
			$cpt = 0;
			foreach( $items as $item ) {
				$thumb = $item->newClassInstance("GalleryPage_Image");
				$normal = $item->newClassInstance("GalleryPage_Image");
				
				if( $this->NormalType == 'OriginalResize' )
					$item->ViewLink = $item->URL;
				else {
					if( method_exists( $normal, "generate{$this->NormalType}" ) )
						$normalImg = $normal->getFormattedImage( $this->NormalType, $this->NormalWidth, $this->NormalHeight );
					else // Fall back to the de facto image resize method 
						$normalImg = $normal->getFormattedImage( 'ResizedImage', $this->NormalWidth, $this->NormalHeight );

					/** If the image was created/got succesfully... **/
					if( $normalImg ) {
						/** Add the URL of the formatted image to our object **/
						$item->ViewLink = Director::baseURL().$normalImg->Filename;
					}
				}
			
				
				if( method_exists( $thumb, "generate{$this->ThumbnailType}" ) )
					$thumbImg = $thumb->getFormattedImage( $this->ThumbnailType, $this->ThumbnailWidth, $this->ThumbnailHeight );
				else // Fall back to the de facto image resize method
					$thumbImg = $thumb->getFormattedImage( 'ResizedImage', $this->ThumbnailWidth, $this->ThumbnailHeight );
				/** If the image can be created... **/			
				$item->ThumbnailURL = Director::baseURL().$thumbImg->Filename;
			}
		}
		
		/** Return the DataObjectSet we've created above **/
		return $items;
	}
	
	
	public function ViewAll() {
		$params = Director::urlParams();	
		return (isset($params['OtherID']) && $params['OtherID'] == 'showall');
	}
	
	public function AlbumView()
	{
		$p = Director::urlParams();
		return $p['Action'] == 'album';
	}
	
	public function UploadView()
	{
		return Director::urlParam('OtherID') == 'upload';
	}
	
	public function CurrentAlbum()
	{
		if($this->current_album)
			return $this->current_album;
		if($this->AlbumView() && $url_segment = Director::urlParam('ID')) {
			return $this->current_album = DataObject::get_one("ImageGalleryAlbum","ImageGalleryPageID = {$this->ID} AND Slug = '" . $url_segment . "'");
		}
		return false;
	}
	
	public function AlbumTitle()
	{
		return $this->CurrentAlbum()->Title;
	}
	

	/**
	 * Check if we're in ViewAll mode. If so, hack the MediaPerPage property to show the total item count.
	 */
	function CurrentGalleryItems() {
		if($this->ViewAll() && $photos = DataObject::get("Image", "ParentID = '{$this->FolderID}'")) {
			$this->MediaPerPage = $photos->Count();
		}
		return parent::CurrentGalleryItems();
	}

	public function checkFolder() {
		if( ! $this->RootFolderID ) {
			$gallery = Folder::findOrMake('ImageGallery');
			$gallery->Title = 'Image Gallery';
			$gallery->write();
			
			if(isset($_POST['URLSegment'])) {
				$folder = Folder::findOrMake('ImageGallery/' . $_POST['URLSegment']);
				$folder->Title = $_POST['Title'];
				$folder->write();
				
				$this->RootFolderID = $folder->ID;
				$this->write();
			}
		}
	}
	
	
	public function onBeforeWrite()
	{
		parent::onBeforeWrite();
		$this->checkFolder();
		if($albums = DataObject::get("ImageGalleryAlbum", "ImageGalleryPageID = {$this->ID}")) {
			foreach($albums as $album) {
				if($folder = DataObject::get_by_id("Folder", $album->FolderID)) {
					$folder->ParentID = $this->RootFolderID;
					$folder->write();
				}
				else {
					$root = DataObject::get_by_id("Folder",$this->RootFolderID);
					$folder = Folder::findOrMake("ImageGallery/{$root->Name}/{$album->Title}");
					$folder->Title = $album->Title;
					$folder->write();
					$album->FolderID = $folder->ID;
					$album->write();
				}
			}
				//File::sync();	
		}
	}
	/*
	public function onBeforeDelete()
	{
		parent::onBeforeDelete();
		if($folder = DataObject::get("Folder","ID = {$this->RootFolderID}"))
			$this->RootFolder()->delete();
	}
	*/
	
	

}

class ImageGalleryPage_Controller extends GalleryPage_Controller
{
	// User authentication is evaluated at the template level to determine the include file.
	public $Authenticated = false;
	
	public function init() 
	{
		parent::init();
		self::prototype2jquery();
		Requirements::css('image_gallery/css/ImageGallery.css');

		if($this->Albums()->Count() == 1)

			$this->current_album = $this->Albums()->First();
		if($this->CurrentAlbum())
			$this->includeUI();

	}
		
	
	private function includeUI()
	{
		switch($this->GalleryUI)
		{
			case "Shadowbox":
				self::includeForShadowbox();
			break;
			case "Lightbox":
				self::includeForLightbox();
			break;
			case "Lightbox (alt)":
				self::includeForLightboxAlt();
			break;
			case "Fancybox":
				self::includeForFancybox();
			break;
			case "Thickbox":
				self::includeForThickbox();
			break;
			case "PrettyPhoto":
				self::includeForPrettyPhoto();
			break;
			case "NyroModal":
				self::includeForNyroModal();
			break;
			

		}
	}

	private static function prototype2jquery() {
		Requirements::clear('gallery/javascript/prototype.js');
		Requirements::clear('gallery/javascript/effects.js');
		Requirements::clear('gallery/javascript/lightwindow.js');
		Requirements::clear('gallery/css/lightwindow.css');
		Requirements::block('jsparty/prototype.js');
		Requirements::javascript('image_gallery/javascript/jquery-1.2.5.min.js'); 
	}

	private static function includeForShadowbox()
	{
		Requirements::javascript('image_gallery/javascript/shadowbox/shadowbox-2.0.js');
		Requirements::javascript('image_gallery/javascript/shadowbox/shadowbox_skin.js');
		Requirements::javascript('image_gallery/javascript/shadowbox/shadowbox_init.js');

		Requirements::css('image_gallery/css/shadowbox/shadowbox.css');
	}

	private static function includeForLightbox()
	{
		Requirements::javascript('image_gallery/javascript/lightbox/jquery.lightbox-0.5.js');
		Requirements::javascript('image_gallery/javascript/lightbox/lightbox_init.js');

		Requirements::css('image_gallery/css/lightbox/jquery.lightbox-0.5.css');
	}
	
	private static function includeForLightboxAlt()
	{
		Requirements::javascript('image_gallery/javascript/lightbox_alt/jquery.lightbox.js');
		Requirements::css('image_gallery/css/lightbox_alt/jquery.lightbox.css');
	}
	private static function includeForThickbox()
	{
		Requirements::javascript('image_gallery/javascript/thickbox/thickbox.js');
		Requirements::css('image_gallery/css/thickbox/thickbox.css');
	}
	
	private static function includeForFancybox()
	{
		Requirements::javascript('image_gallery/javascript/fancybox/jquery.fancybox-1.0.0.js');
		Requirements::javascript('image_gallery/javascript/fancybox/jquery.pngFix.pack.js');
		Requirements::javascript('image_gallery/javascript/fancybox/fancybox_init.js');

		Requirements::css('image_gallery/css/fancybox/fancy.css');
	}
	private static function includeForPrettyPhoto()
	{
		Requirements::javascript('image_gallery/javascript/prettyphoto/jquery.prettyPhoto.js');
		Requirements::javascript('image_gallery/javascript/prettyphoto/prettyphoto_init.js');

		Requirements::css('image_gallery/css/prettyphoto/prettyPhoto.css');
	}
	
	private static function includeForNyroModal()
	{
		Requirements::javascript('image_gallery/javascript/nyro_modal/jquery.nyroModal-1.3.0.js');
		Requirements::javascript('image_gallery/javascript/nyro_modal/nyro_modal_init.js');

		Requirements::css('image_gallery/css/nyro_modal/nyroModal.css');
	
	}
	
		
	public function ManagementSidebar() {
		if(Permission::check('ADMIN')){
			$this->Authenticated = true;
			
			self::prototype2jquery();				
			Requirements::javascript('image_gallery/javascript/ui/ui.js');
			Requirements::javascript('image_gallery/javascript/ui/dialog.js');
			Requirements::javascript('image_gallery/javascript/ui/image_gallery_admin.js');

			$UploadLink = $this->Link('upload');
			$ViewAllLink = $this->Link('showall');
			return $this->customise(array('UploadLink' => $UploadLink, 'ViewAllLink' => $ViewAllLink))->renderWith('ImageGallery_ManagementSidebar');
		}
    }

	public function HideAdminView()	{	
		$params = Director::urlParams();	
		return (isset($params['OtherID']) && $params['OtherID'] == 'hide');
	}
	
	public function AdminGallery()
	{
		return $this->Authenticated && !$this->HideAdminView();
	}
	
	private function isShadowbox()
	{
		return $this->GalleryUI == 'Shadowbox';
	}
	
	private function isLightbox()
	{
		return $this->GalleryUI == 'Lightbox';
	}

	/** 
	 * Easy way to log in and get redirected back to the gallery with privileges. Just go to /admin.
	 */
   public function admin() {
    	if(!Permission::check('ADMIN')) {
				Security::permissionFailure($this,
					"You must be logged in to manage the gallery."
				);
    	}
    	else {
    		Director::redirect($this->Link());
    	}
    
   }
       
   public function gallerydelete() {
			if($image = DataObject::get_by_id("Image", Director::urlParam('ID'))) {
				$image->delete();
			}
    }
    
	/**
	 * Returns a caption to the editing dialog box along with a link that tells the form where to submit.
	 */
   public function gallerygetcaption() {
		if($image = DataObject::get_by_id("Image", Director::urlParam('ID'))) {
				$image->flushCache();
				$link = Director::currentURLSegment() . 'galleryupdatecaption/' . $image->ID;
				$caption = $image->Content;
				return $this->customise(array('link' => $link,'caption' => $caption))->renderWith('ImageGalleryCaptionForm');
		}
    }
   
 /** 
	* There's proabably a better way to do this. We could at least sanitize the raw form data.
	*/
   public function galleryupdatecaption() {
			if($image = DataObject::get_by_id("Image", Director::urlParam('ID'))) {
				$image->Content = $_POST['caption'];
				$image->write();
			}
   }
	
	/**
	 * Not the most efficient sorting method. Runs a query for every photo on the page.
	 */
    public function gallerysort() {
			foreach($_POST['item'] as $sort => $id) {
				if($image = DataObject::get_by_id("Image",$id)) {
					$image->Sort = $sort;
					$image->write();
				}
			}
    }
    
   /**
    * This is the script that gets run per the SWFUploadField configuration. The name "swfupload_file"
    * comes from SWFUploadConfig class. "file_destination_path" is also set there.
    * To prevent duplicates, each file is prefixed with a timestamp.
    * At the end, we echo out the ID of the image that was saved, and that gets appened to an array
    * of hidden inputs on the frontend that feeds the subsequent caption form.
    */     
    public function handleswfupload() {
		if (isset($_FILES["swfupload_file"]) && is_uploaded_file($_FILES["swfupload_file"]["tmp_name"])) {
				$album = DataObject::get_by_id("ImageGalleryAlbum", $_POST['AlbumID']);
				if($max = DataObject::get_one("Image","ParentID = " . $album->FolderID, false, "Sort DESC"))
					$sort = $max->Sort + 1;
				else
					$sort = 1;
				$root = $this->RootFolder()->Name;
				$file = new Image();
				$file->AlbumID = $album->ID;
				$file->Sort = $sort;
				$file->loadUploaded($_FILES["swfupload_file"], "ImageGallery/{$root}/{$album->Name}");
				$file->setField('ParentID',$album->FolderID);
				$file->write();

				echo $file->ID; // pass the file back to the form
		} else {
			echo ' '; // return something or SWFUpload won't fire uploadSuccess
		}			
    }
    
    public function handleselectedphotos()
    {
		if(!empty($_POST['move']) && is_array($_POST['selected_photos'])) {
			$new_album = DataObject::get_by_id("ImageGalleryAlbum",$_POST['move']);
			foreach($_POST['selected_photos'] as $id) {
				$image = DataObject::get_by_id("File",$id);
				$image->AlbumID = $new_album->ID;
				$image->ParentID = $new_album->FolderID;
				$image->write();
			}
		}
		Director::redirect(DataObject::get_by_id("ImageGalleryAlbum",$_POST['CurrentAlbumID'])->Link());
    }
    
    public function ReturnPhotos($data,$form) {
				return $this->renderWith(array('ImageGalleryPage_ReturnPhotos','Page'));
    }
    
    /**
     * The array "uploaded_files" is a series Image IDs dynamically generated by SWFUploadField as
     * the uploads complete. The method uses this list to build out textareas for each uploaded image.
     * There's a lot of markup jammed in here to create the interface. Not the pretties thing in the world,
     * but the only way I know of.
     */    
    public function CaptionForm() {
		Requirements::javascript('image_gallery/javascript/ui/caption_form.js');
		Requirements::clear('swfupload/javascript/swfu.js');
		Requirements::block('swfupload/javascript/swfu.js');				
		if(isset($_POST['uploaded_files']) && is_array($_POST['uploaded_files'])) {
			$list = implode("','",$_POST['uploaded_files']);
			$uploaded_images = DataObject::get("Image","File.ID IN ('" . $list . "')");
		}
		else {
			$uploaded_images = new DataObjectSet();
		}
		$fieldset = new FieldSet();
		$fieldset->push(new HiddenField('AlbumID','',$_POST['AlbumID']));
		$fieldset->push(new LiteralField('ul','<ul id="caption-list">'));
		foreach($uploaded_images as $image) {
			$fieldset->push(new LiteralField('li' . $image->ID, '<li>'));
			$fieldset->push(new LiteralField('div','<div class="caption-image">'));
			$fieldset->push(new LiteralField('img' . $image->ID, '<img src="' . $image->SetWidth(100)->URL . '" alt="" />'));
			$fieldset->push(new LiteralField('div','</div>'));
			$fieldset->push(new TextareaField('caption_'.$image->ID,'',5,40,'Enter caption...'));
			$fieldset->push(new LiteralField('li','</li>'));
		}
		$fieldset->push(new LiteralField('ul','</ul>'));
		$form = new Form($this, 'CaptionForm', $fieldset, new FormAction('SaveCaptions','Save'));
		return $form;
    }
    
	/**
	 * All the caption fields come in with the format "caption_$ImageID".
	 */
    public function SaveCaptions($data,$form) {
		foreach($_POST as $k => $v) {
			$id = str_replace('caption_','',$k);
			if(is_numeric($id)) {
				if($image = DataObject::get_by_id("Image",$id)) {
					if($v !== "Enter caption...") {
						$image->Content = $v;
						$image->write();
					}
				}
			}
		}
		$this->flushCache();
		$album = DataObject::get_by_id("ImageGalleryAlbum",$data['AlbumID']);
		Director::redirect($album->Link());
    }
  /**
   * Show the SWFUploadForm
   */  
   public function UploadForm() {
    	if(!Permission::check('ADMIN')) {
			Security::permissionFailure($this,
				"Uploading photos is an administrator task. Please log in.");
    	}
    	
    	else {
			if(!$this->Albums())
				return "There are no albums in this Image Gallery. Please create some in the CMS before uplaoding.";

    		$fields = new FieldSet();
    		$album_id = isset($_POST['AlbumID']) ? $_POST['AlbumID'] : $this->CurrentAlbum()->ID;
	    	$fields->push(new HiddenField('AlbumID','',$album_id));
			if(!isset($_POST['uploaded_files'])) {
	    		// Allowed filetypes. Will grey out all others when the dialog box renders.
	    		SWFUploadConfig::addFileType('jpg');
	    		SWFUploadConfig::addFileType('jpeg');
	    		if($this->CurrentAlbum()) {
		    		$album_id = $this->CurrentAlbum()->ID;
		    		SWFUploadConfig::addPostParam('AlbumID',$album_id);
		    		$fields->push(new HeaderField($title = 'Upload photos to "'.$this->CurrentAlbum()->Title.'"',$headingLevel='3'));
		    	}
	
				$fields->push(		
					// Gets an array of configuration vars. Can also be set through static calls to SWFUploadConfig,
					// as seen above.
					new SWFUploadField("UploadForm","","",array(
						'file_upload_limit' => '100', // how many files can be uploaded
						'file_queue_limit' => '100', // how many files can be in the queue at once
						'browse_button_text' => 'Choose photos...',
						'upload_url' => $this->Link('handleswfupload'), // what php script will be run after upload
						'file_destination_path' => $this->Folder()->Filename, // where do the files get copied to
						'required' => 'true' // will hide the submit button until valid files are put in the queue.
					))
	    		);
    		}	
    		return new Form($this, "UploadForm", 
    			$fields,
    			new FieldSet(
    				new FormAction('ReturnPhotos', 'Save')
    			)
    		);
    	}
    }
    
    public function MovePhotos()
    {
    	if($this->Albums()) {
			return new FieldGroup(
				new LabelField(null,'Move selected photos to   '),
				new DropdownField('move','',$this->Albums("ImageGalleryAlbum.ID != '" . $this->CurrentAlbum()->ID . "'")->toDropdownMap(),'',null,'-- Please select --'),
				new HiddenField('CurrentAlbumID','',$this->CurrentAlbum()->ID)
			);
    	}
    	return null;
    }
}


?>